// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - LOBBY [PERKS]
// Perks list
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// Starting X and Y of the list items
#define PMENU_STARTX					6.0
#define PMENU_STARTY					32.0

// Move down this many pixels when drawing items
#define PMENU_LEFTTOP					8.0

// Dimensions of left box on perk menu
#define PMENU_LEFTWIDTH					214.0
#define PMENU_LEFTHEIGHT				517.0

// Amount of pixels between each item
#define PMENU_LISTPAD					4.0

// How many perks can the menu visually hold at once?
#define PMENU_MAXITEMS					13

struct PerkMenuItem
{
	int Index;					// REAL index this points to
};

namespace LBPerks
{
	using LBCore;
	using ZTMaterials;
	
	private bool MidGameSelect;				// This is NOT a lobby menu, it's mid-game
	
	private struct Rect BackgroundRect;		// Where to draw BG
	bool DirtyBackground;					// BG needs redrawing
	
	private struct Rect ItemRect;			// Bounding box for item list
	bool DirtyItems;						// Items need redrawing
	
	private int HoverPerk = -1;				// Which perk are we hovering over?
	
	// Some perks are hidden, and perk 0 is nothing!
	// As a result, creating a temporary list is just a bit easier
	
	private struct PerkMenuItem PMItem[PMENU_MAXITEMS];
	private int PMItemCount;
	
	// WERE we sending a perk request?
	// If this mismatches, then re-draw our items
	private bool SendingRequest;
	
	// ------------------------/
	// Initialize!
	// ------------------------/
	
	void LBP_Init(bool MGS = false)
	{
		MidGameSelect = MGS;
		
		if (MGS)
			LocalAmbientSound("perkselect/open", 127);
		
		// Generate elements
		LBP_Generate();
	}
	
	// ------------------------/
	// Cleanup
	// ------------------------/
	
	void LBP_Cleanup()
	{
		ClearHUDMessage(HID_PERK_BG);
		
		// Clear items
		int HID = HID_PERK_ITEMS;
		for (int i=0; i<PMENU_MAXITEMS; i++)
		{
			// BG
			ClearHUDMessage(HID);
			
			// Name
			ClearHUDMessage(HID-1);
			
			HID -= 2;
		}
	}
	
	// ------------------------/
	// Draw
	// ------------------------/
	
	void LBP_Draw()
	{
		if (LobbyMode != LM_PERKS)
			return;
			
			
		// Background!
		if (DirtyBackground)
		{
			DirtyBackground = false;
			HUDMessageMaterial(PSELBG, BackgroundRect, HID_PERK_BG);
		}
		
		// Items!
		if (DirtyItems)
		{
			DirtyItems = false;
			
			int CPN = ConsolePlayerNumber();
			
			int curY = ItemRect.Y + PMENU_LEFTTOP;
			int HID = HID_PERK_ITEMS;
			
			for (int i=0; i<PMENU_MAXITEMS; i++)
			{
				if (!LBP_ValidItem(i))
				{
					ClearHUDMessage(HID);
					ClearHUDMessage(HID-1);
					HID -= 2;
					continue;
				}
				
				int PI = PMItem[i].Index;
				
				// Draw the item
				
				// WAITING ON A PERK REQUEST
				if (Replication.SendingPerkChange)
					HUDMessageMaterialOnly(HoverPerk == i ? PSTAB6 : PSTAB5, ItemRect.X, curY, HID); 
				else if (PlayerState[CPN].PerkIndex == PI)
					HUDMessageMaterialOnly(HoverPerk == i ? PSTAB4 : PSTAB3, ItemRect.X, curY, HID); 
				else
					HUDMessageMaterialOnly(HoverPerk == i ? PSTAB2 : PSTAB1, ItemRect.X, curY, HID); 
					
				HID --;
				
				// The name
				int nameY = curY + FixedMul(PSTAB1.H, 0.5);
				SetFont(FNT_PLAIN);
				HUDMessageUnscaled(ItemRect.X + 8.0, nameY, PerkList[PI].DisplayName, ALIGN_LEFT, ALIGN_CENTER, HID);
				HID --;
				
				curY += PSTAB1.H + PMENU_LISTPAD;
			}
		}
	}
	
	// ------------------------/
	// Tick
	// ------------------------/
	
	void LBP_Tick()
	{
		if (LobbyMode != LM_PERKS)
			return;
			
		// Our sendrequest-state is mismatched! Re-draw the items
		if (SendingRequest != Replication.SendingPerkChange)
		{
			SendingRequest = Replication.SendingPerkChange;
			DirtyItems = true;
		}
			
		// Decide what we're hovering over
		int LastHover = HoverPerk;
		
		HoverPerk = -1;
		
		int mX = GetMouseX();
		int mY = GetMouseY();
		
		// Our mouse is within horizontal bounds
		if (mX >= ItemRect.X && mX <= ItemRect.X + ItemRect.W)
		{
			int curY = ItemRect.Y + PMENU_LEFTTOP;
			for (int i=0; i<PMENU_MAXITEMS; i++)
			{
				// We're hovering over this item!
				if (mY >= curY && mY <= curY + PSTAB1.H && LBP_ValidItem(i))
				{
					HoverPerk = i;
					break;
				}
				
				// Not hovering, continue on
				curY += (PSTAB1.H + PMENU_LISTPAD);
			}
		}
		
		// Our hover value ACTUALLY changed!
		if (LastHover != HoverPerk)
		{
			LastHover = HoverPerk;
			DirtyItems = true;
			
			AmbientSound("trader/perkover", 127);
		}
	}
	
	// ------------------------/
	// Generate elements
	// ------------------------/
	
	void LBP_Generate(bool ifShown = false)
	{
		int cenX, cenY;
		
		if (ifShown && LobbyMode != LM_PERKS)
			return;
			
		DirtyBackground = true;
		DirtyItems = true;
		
		// Decide where to draw our background!
		
		// Lobby menu
		if (!MidGameSelect)
		{
			cenX = LBCore::GetLobbyCenterX();
			cenY = LBCore::GetLobbyCenter();
		}
		else
		{
			cenX = CenterX() * 1.0;
			cenY = CenterY() * 1.0;
		}
		
		cenX -= FixedMul(PSELBG.W, 0.5);
		cenY -= FixedMul(PSELBG.H, 0.5);
		
		SetRect(BackgroundRect, cenX, cenY, PSELBG.W, PSELBG.H);
		
		// Set item bounding box
		int itX = cenX + PMENU_STARTX;
		int itY = cenY + PMENU_STARTY;
		
		SetRect(ItemRect, itX, itY, PMENU_LEFTWIDTH, PMENU_LEFTHEIGHT);
		
		// "Re-filter" our perks
		LBP_Populate();
	}
	
	// ------------------------/
	// Is this item hover-able / draw-able?
	// ------------------------/
	
	bool LBP_ValidItem(int idx)
	{
		// Extends past filtered items
		if (idx >= PMItemCount)
			return false;
			
		return true;
	}
	
	// ------------------------/
	// Populate / Filter our Items
	// ------------------------/
	
	void LBP_Populate()
	{
		PMItemCount = 0;
		
		for (int i=0; i<PerkListSize; i++)
		{
			// Hidden perk, ignore
			if (PerkList[i].Hidden)
				continue;
				
			PMItem[PMItemCount].Index = i;
			PMItemCount ++;
		}
	}
	
	// ------------------------/
	// Click!
	// ------------------------/
	
	void LBP_MouseClick()
	{
		if (LobbyMode != LM_PERKS)
			return;
		
		if (HoverPerk < 0)
			return;
			
		// Already sending a perk, be patient!
		if (Replication.SendingPerkChange)
			return;
			
		AmbientSound("trader/perkclick", 127);
			
		// Set replication junk, etc. first
		AttemptPerkChange(PMItem[HoverPerk].Index, !MidGameSelect);
		
		SendingRequest = Replication.SendingPerkChange;
		
		// Dirty the items! Grey them out, we're waiting
		DirtyItems = true;
	}
}