// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - LOBBY [TOP]
// Handles drawing the top category bar for the lobby
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#define LOBBYTAB_PADDING				16.0

#define PERKFLASH_COUNT					10
#define PERKFLASH_DELAY					3

namespace LBTop
{
	using ZTMaterials;
	using LBCore;
	
	private bool DirtyBar;					// Bar needs to be redrawn
	private bool DirtyOverview;				// Overview needs redrawn
	private bool DirtyPerks;				// Perks need redrawn
	
	private bool HoverOverview;
	private bool HoverPerks;
	
	private int ResX, ResY;					// Resolution
	
	private int BarX, BarY;					// Bar position
	private int BarWidth;					// Bar width
	
	private struct Rect OverviewRect;		// Overview button 
	private struct Rect PerksRect;			// Perks button 
	
	private int TopTick;					// For PERK ERROR
	private int TopFlashCount;
	private bool TopFlash;
	
	// ------------------------/
	// Initialize!
	// ------------------------/
	
	private void LT_Init()
	{
		LT_Populate();
	}
	
	// ------------------------/
	// Draw!
	// ------------------------/
	
	private void LT_Draw()
	{
		// Draw the bar
		if (DirtyBar)
		{
			DirtyBar = false;
			HUDMessageRectangleHor(BarX, BarY, BarWidth, HID_LOBBY_TOPBAR, LOBBTOPL, LOBBTOPM, LOBBTOPR);
		}
		
		// Overview button
		if (DirtyOverview)
		{
			DirtyOverview = false;
			
			HUDMessageMaterial((LobbyMode == LM_Overview) ? LOBBOVR3 : (HoverOverview ? LOBBOVR2 : LOBBOVR1), OverviewRect, HID_LOBBY_OVERVIEW);
		}
		
		// Perk button
		if (DirtyPerks)
		{
			DirtyPerks = false;
			HUDMessageMaterial((LobbyMode == LM_PERKS) ? LOBBPRK3 : (HoverPerks ? LOBBPRK2 : LOBBPRK1), PerksRect, HID_LOBBY_PERKS);
			
			if (TopFlash)
				HUDMessageMaterial(LOBBPRKZ, PerksRect, HID_LOBBY_PERKERROR);
			else
				ClearHUDMessage(HID_LOBBY_PERKERROR);
		}
	}
	
	// ------------------------/
	// Tick logic
	// ------------------------/
	
	private void LT_Tick()
	{
		bool HOV = Mousing(OverviewRect);
		bool HPK = Mousing(PerksRect);
		
		// Handle perk error flash
		if (TopTick > 0)
		{
			TopTick ++;
			
			if (TopTick >= PERKFLASH_DELAY+1)
			{
				TopTick = 1;
				TopFlashCount ++;
				DirtyPerks = true;
				
				if (TopFlashCount >= PERKFLASH_COUNT)
				{
					TopFlash = false;
					TopTick = 0;
					TopFlashCount = 0;
				}
				else
				{
					TopFlash = !TopFlash;
					if (TopFlash)
						AmbientSound("trader/perkover", 127);
				}
			}
		}
		
		if (HoverOverview != HOV)
		{
			HoverOverview = HOV;
			DirtyOverview = true;
			
			if (HOV)
				AmbientSound("trader/wepover", 127);
		}
		
		if (HoverPerks != HPK)
		{
			HoverPerks = HPK;
			DirtyPerks = true;
			
			if (HPK)
				AmbientSound("trader/wepover", 127);
		}
	}
	
	// ------------------------/
	// Cleanup
	// ------------------------/
	
	private void LT_Cleanup()
	{
		// Cleanup the bars
		HUDMessageRectangleHor(BarX, BarY, BarWidth, HID_LOBBY_TOPBAR, LOBBTOPL, LOBBTOPM, LOBBTOPR, true);
		
		// Buttons
		ClearHUDMessage(HID_LOBBY_OVERVIEW);
		ClearHUDMessage(HID_LOBBY_PERKS);
	}

	// ------------------------/
	// Actually POPULATE this with elements
	// ------------------------/
	
	private void LT_Populate()
	{
		// Sync res
		ResX = ResolutionX();
		ResY = ResolutionY();
		
		BarX = 0.0;
		BarY = 64.0;
		BarWidth = ResX * 1.0;
		
		DirtyBar = true;
		DirtyOverview = true;
		DirtyPerks = true;
		
		// Set button positions
		int butX = BarX + 21.0;
		int butY = BarY + 24.0;
		int curX = butX;
		
		// -- OVERVIEW
		SetRect(OverviewRect, curX, butY, LOBBOVR1.W, LOBBOVR1.H);
		curX += LOBBOVR1.W + LOBBYTAB_PADDING;
		
		// -- PERKS
		SetRect(PerksRect, curX, butY, LOBBPRK1.W, LOBBPRK1.H);
		curX += LOBBPRK1.W + LOBBYTAB_PADDING;
	}
	
	// ------------------------/
	// Dirty our tabs up!
	// ------------------------/
	
	private void LT_DirtyTabs()
	{
		DirtyOverview = true;
		DirtyPerks = true;
	}
	
	// ------------------------/
	// Left click
	// ------------------------/
	
	private void LT_MouseClick()
	{
		// -- OVERVIEW
		if (HoverOverview)
		{
			AmbientSound("trader/wepclick", 128);
			LBCore::L_SwitchModes(LM_OVERVIEW);
		}
			
		// -- PERKS
		else if (HoverPerks)
		{
			AmbientSound("trader/wepclick", 128);
			LBCore::L_SwitchModes(LM_PERKS);
		}
	}
	
	// ------------------------/
	// Flash perk error
	// ------------------------/
	
	void LT_DoPerkError()
	{
		TopTick = 1;
		TopFlash = false;
		TopFlashCount = 0;
	}
}