// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==
// S C R O L L B A R S
// Handy for list things!
// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==

// Scrollbars will use this many ID's to clean themselves up
// (Please space them apart by at least this much to avoid conflicts)

#define SCROLLBAR_CLEANUPSIZE				50

struct ScrollBarT
{
	struct Rect Bounds;			// Bounds for this particular bar
	
	int Pct;
	int BarY, LastBarY;			// USed for checking dirty state
	int ID;						// HID to use for drawing
	bool Disabled;				// Is this bar disabled?
};

// Set up a scrollbar struct!
void SetScrollbar(ScrollBarT& Bar, struct Rect& Boundaries, int ID = -1)
{
	Bar.BarY = Boundaries.Y;
	Bar.LastBarY = Boundaries.Y+1;
	Bar.Bounds.X = Boundaries.X;
	Bar.Bounds.Y = Boundaries.Y;
	Bar.Bounds.W = Boundaries.W;
	Bar.Bounds.H = Boundaries.H;
	
	if (ID >= 0)
		Bar.ID = ID;
} 

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

namespace Scrollbars
{
	using ZTMaterials;
	
	bool barHeld;			// The bar is being held down!
	
	private ScrollBarT 		NullBar;
	private ScrollBarT& 	StoredBar = NullBar;
	
	// ScrollBarT& GetBar() {}
	
	// Is a scrollbar even required?
	bool BarNeeded(int total, int maximum)
	{
		return (total > maximum);
	}
	
	// First index to start drawing items from
	// This should extend to the end of the list, but
	// any items past the list should not be drawn
	int ScrollIndex(int total, int maximum, ScrollBarT& Bar)
	{
		int LMax = total - maximum;
		return FixedMul(LMax, Bar.pct);
	}
	
	// Does this scrollbar need to be redrawn?
	bool ScrollbarDirty(ScrollBarT& Bar)
	{
		if (Bar.LastBarY != Bar.BarY)
		{
			Bar.LastBarY = Bar.BarY;
			return true;
		}
		
		return false;
	}
	
	// Draw a scrollbar!
	void Bar_Draw(ScrollBarT& Bar)
	{
		int HID = HUDMessageRectangle(Bar.Bounds.X, Bar.Bounds.Y, Bar.Bounds.H, Bar.ID, TRADSBR1, TRADSBR2, TRADSBR3);
		
		if (!Bar.Disabled)
			HUDMessageMaterialOnly(TRADSBAR, Bar.Bounds.X + Bar.Bounds.W, Bar.BarY, HID, ALIGN_RIGHT, ALIGN_CENTER);
	}
	
	// Cleanup a scrollbar
	void Bar_Cleanup(ScrollBarT& Bar)
	{
		for (int l=0; l<SCROLLBAR_CLEANUPSIZE; l++)
			ClearHUDMessage(Bar.ID - l);
	}
	
	// Reset the bar location
	void Bar_Reset(ScrollBarT& Bar)
	{
		Bar.Pct = 0.0;
		Bar.BarY = Bar.Bounds.Y;
	}
	
	// Every tic
	void Bar_Tic()
	{
		if (barHeld)
		{
			// Calculate a percentage
			int MY = clamp(GetMouseY(), StoredBar.Bounds.Y, StoredBar.Bounds.Y + StoredBar.Bounds.H);
			int pct = FixedDiv(MY - StoredBar.Bounds.Y, StoredBar.Bounds.H);
			
			StoredBar.Pct = Pct;
			StoredBar.BarY = MY;
		}
	}

	// Clicked
	void Bar_Click()
	{
		// ScrollBarT& Bar = GetBar();
		
		// if (Mousing(Bar.Bounds))
		// {
			// StoredBar = Bar;
			// barHeld = true;
		// }
	}
	
	// Released
	void Bar_Release()
	{
		barHeld = false;
		StoredBar = NullBar;
	}
	
	// Try to set a bar, only if it's moused over
	void Bar_AttemptSet(struct ScrollBarT& Bar)
	{
		if (Bar.Disabled)
			return;
			
		if (Mousing(Bar.Bounds))
		{
			StoredBar = Bar;
			barHeld = true;
		}
	}
	
	// Is a bar being hovered over?
	bool Bar_Hovered(struct ScrollBarT& Bar)
	{
		return (Mousing(Bar.Bounds));
	}
	
	void Bar_Enable(struct ScrollBarT& Bar) { Bar.Disabled = false; }
	void Bar_Disable(struct ScrollBarT& Bar) { Bar.Disabled = true; }
}