// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==
// T O O L T I P S
// Controls tooltip-related things
// --==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==--==

// Distance from the cursor
#define TOOLTIP_PADDING_R			32.0;
#define TOOLTIP_PADDING_L			24.0;

// Used for line wrapping
#define TOOLTIP_LINESIZE			270.0;

struct TooltipT
{
	bool Dirty;					// It needs to be redrawn
	bool Visible;				// Is this tooltip visible?
	int X, Y;					// X and Y coords IN PIXELS
	int HID;					// HID to use for this tooltip
	
	str Title;					// Title to show for the menu
	str Description;			// Description to show
	str Pic;					// Image to show
	
	int WrappedLines;			// Number of lines from our auto-wrap
	
	struct Picture TopPic;		// Top picture
	struct Picture MidPic;		// Middle picture
	struct Picture BotPic;		// Bottom picture
};

namespace Tooltips
{
	// Set tooltip information
	void TT_Set(struct TooltipT& Tip, str tTitle, str tDescription, int HID = -1, str tPic = "")
	{
		Tip.Title = tTitle;
		
		// SET THE WRAPPED DESCRIPTION
		Tip.Description = StrWrap(tDescription, TOOLTIP_LINESIZE, GLYPHWIDTH_PLAIN);
		Tip.WrappedLines = GameState.HUD.LastWrappedLines;
		
		if (tPic != "")
			Tip.Pic = tPic;
		
		if (HID >= 0)
			Tip.HID = HID;
	}
	
	// Set theming information for the tooltip
	void TT_SetTheme(struct TooltipT& Tip, struct Picture& TP, struct Picture& MD, struct Picture& BM)
	{
		CopyPicture(TP, Tip.TopPic);
		CopyPicture(MD, Tip.MidPic);
		CopyPicture(BM, Tip.BotPic);
	}
	
	// Show or hide a tooltip
	void TT_Show(struct TooltipT& Tip, bool showVal = true)
	{
		if (Tip.Visible == showVal)
			return;
			
		Tip.Visible = showVal;
		Tip.Dirty = true;
	}
	
	// Execute a tick on a tooltip
	void TT_Tick(struct TooltipT& Tip)
	{
		if (!Tip.Visible)
			return;
			
		// Did our mouse position change from where it was before?
		int mX = GetMouseX();
		int mY = GetMouseY();
		
		if (mX != Tip.X || mY != Tip.Y)
		{
			Tip.X = mX;
			Tip.Y = mY;
			Tip.Dirty = true;
		}
	}
	
	// Draw a tooltip
	void TT_Draw(struct TooltipT& Tip)
	{
		if (!Tip.Dirty)
			return;
			
		Tip.Dirty = false;
		
		// How tall?
		int height = 256.0;
		
		// Get the width of the tip
		int width = Tip.TopPic.W;
		
		int ttX = Tip.X + TOOLTIP_PADDING_R;
		int tty = Tip.Y;
		
		// If our mouse is on the right half of the screen, then draw
		// the tooltip to the left of the cursor
		
		if (GetMouseX() >= (CenterX()*1.0))
			ttX = (Tip.X - width) - TOOLTIP_PADDING_L;
			
		// Keep it from going off screen
		int ResY = ResolutionY() * 1.0;
		if (tty+height >= ResY)
			tty = ResY - height;
			
		// -- DRAW THE BACKGROUND FOR IT
		int newHID = HUDMessageRectangle(ttX, ttY, height, Tip.HID, Tip.TopPic, Tip.MidPic, Tip.BotPic, !Tip.Visible);
		
		int ttcen = ttX + FixedMul(width, 0.5);
		
		// -- DRAW THE ITEM NAME
		SetFont(FNT_TOOLTIPTITLE);
		HUDMessageUnscaled(ttcen, ttY + 150.0, Tip.Visible ? Tip.Title : " ", ALIGN_CENTER, ALIGN_TOP, newHID);
		newHID --;
		
		// -- DRAW THE ITEM DESCRIPTION
		SetFont(FNT_TOOLTIPDESC);
		HUDMessageUnscaled(ttcen, ttY + 190.0, Tip.Visible ? Tip.Description : " ", ALIGN_CENTER, ALIGN_TOP, newHID);
		newHID --;
		
		// -- DRAW THE PICTURE
		SetFont(Tip.Pic);
		HUDMessageUnscaled(ttcen, ttY + 86.0, Tip.Visible ? "a" : " ", ALIGN_LEFT, ALIGN_TOP, newHID);
	}
}