// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - HUD
// Contains functions related to drawing HUD elements
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#define CIRCLE_PADDING			120.0

#define MUSIC_SEGMENTINCREASE	2
#define MUSIC_SEGMENTLENGTH		16
#define MUSIC_SEGMENTTICS		1
#define MUSIC_FADEGOAL			3
#define MUSIC_SHOWTIME			0.0
#define MUSIC_FADETIME			0.15

#define MUSIC_SONGX				8.0
#define MUSIC_SONGY				24.0
#define MUSIC_ARTISTX			16.0
#define MUSIC_ARTISTY			40.0

#define MUSIC_INFO_FADEIN		0.25
#define MUSIC_INFO_HOLD			3.0
#define MUSIC_INFO_FADEOUT		1.0

#define MUSIC_STARTX			32.0
#define MUSIC_STARTY			128.0

#define AMMONADE_X				149.0
#define AMMONADE_Y				70.0

#define AMMOPRIM_X				235.0
#define AMMOPRIM_Y				93.0

#define AMMOSEC_X				278.0
#define AMMOSEC_Y				48.0

#define FIREMODE_X				150.0
#define FIREMODE_Y				105.0

#define PERKICON_X				80.0
#define PERKICON_Y				90.0

// How often should we update perk index junk? In tics
#define PERKSYNC_INTERVAL		35

// Contains information about numbers, etc. that will be drawn on the screen
struct ZT2HUDInfo
{
	int TimeLeft;					// Actual time left
	int ZombiesLeft;				// Zombies we NEED to kill
	
	int MouseX;						// X position of mouse
	int MouseY;						// Y position of mouse
	
	int LastWrappedLines;			// Stores the last output from the StrWrap command
	
	bool PerkSelectIsOpen;			// Perk select menu is open
	bool TraderIsOpen;				// Trader menu is being shown
	bool LobbyIsOpen;				// The menu IS open
	bool ClosedLobby;				// Our lobby menu was closed at least once
	bool WasReady;					// Were we READY the last tick?
	
	bool SquadSyncPending;			// Received player state update, lobby's SQUADS needs sync
	
	str CurrentSong;				// Song we're playing
	str CurrentArtist;				// Artist of the song
	
	bool NeedsBinding;				// Keys need binding! Handled by ZTInput.h
	str BindingString;				// Handled by ZTInput.h
};

namespace ZT2HUD
{
	using LBCore;
	using ZTMaterials;
	
	// Last known resolution values
	int ResX, ResY;
	
	private struct Rect CompassRect;				// For compass
	private bool DrawCompass;						// Should we draw the compass?
	
	private bool DrawRealHUD;						// Drawing the "real" HUD
	private bool DirtyRealState;					// Real HUD state has changed
	
	private int LastPerk;							// Last perk we had selected
	private int LastGrenadeIndex;					// WeaponList index for nade
	private int LastWeaponIndex;					// Last weapon we had selected
	private int PerkSyncTick;
	private str LastFMImage;						// Last FireMode image
	
	private bool DirtyPerk;							// Our perk index is dirty
	private bool DirtyFireMode;						// Re-draw firemode?
	private bool DirtyGrenade;						// Re-draw grenade number?
	private bool DirtyAmmo;							// Re-draw ammo numbers?
	private bool DrawAmmoNumbers;					// Should we actually draw them?
	
	private int Ammo_LastGrenade;					// How many nades we HAD
	private int Ammo_LastPrimary;					// How much primary we HAD
	private int Ammo_LastSecondary;					// How much secondary we HAD

	// ------------------------/
	// SERVER -> CLIENT
	// Create the non-persistent HUD
	// ------------------------/

	script "RequestHUDCreation" CLIENTSIDE
	{
		if (WrongClient())
			terminate;
			
		ACS_NamedExecuteAlways("CreateZT2HUD", 0, false);
	}

	// ------------------------/
	// Mainline hook for ZT2 HUD
	// ------------------------/

	script "CreateZT2HUD" (int persistent)
	{
		// Persistent elements are things that ALWAYS show
		// Spectators will also see these!
		
		if (persistent)
		{
			// Top-right circle
			ACS_NamedExecute("HUD_BioCircleController",0,0,0,0);
			ACS_NamedExecute("HUD_PersistentControl",0,0,0,0);
		}
		
		// For REAL players only
		else
		{
			ACS_NamedExecute("HUD_WaveCountdownController",0,0,0,0);
		}
	}

	// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

	// ------------------------/
	// Controls the circle in the top-right
	// ------------------------/

	script "HUD_BioCircleController"
	{
		// The HUD elements need to be re-drawn
		bool dirty = true;
		
		int lastNumber, lastGoal, lastState, lastWave;
		bool traderInProgress;
		bool shouldFormat;
		
		while (true)
		{
			// THE ACTUAL GAME STATE!
			if (lastState != GameState.State)
			{
				dirty = true;
				lastState = GameState.State;
			}
			
			// WAVE CHANGED
			if (lastWave != GameState.Wave)
			{
				dirty = true;
				lastWave = GameState.Wave;
			}
				
			// Actual numbers
			int checkNumber = (GameState.State == STATE_WAVE) ? GameState.HUD.ZombiesLeft : GameState.HUD.TimeLeft;
			if (lastNumber != checkNumber)
			{
				lastNumber = checkNumber;
				dirty = true;
				
				shouldFormat = (GameState.State == STATE_TRADER);
			}
			
			// Trader state
			if (GameState.State == STATE_TRADER && !traderInProgress)
			{
				traderInProgress = true;
				dirty = true;
			}
			else if (GameState.State != STATE_TRADER && traderInProgress)
			{
				traderInProgress = false;
				dirty = true;
			}
			
			if (dirty)
			{
				dirty = false;
				
				// Should we even show the circle?
				if (GameState.State != STATE_LOBBY && GameState.State != STATE_COUNTDOWN)
				{
					bool isBoss = IsBossWave();

					// (fixed xPos, fixed yPos, str text, int alX, int alY, int id, fixed sTime = 3.0, fixed fTime = 0.0)
					SetFont(traderInProgress ? "BIOTIME1" : "BIOTIME2");
					
					if (isBoss)
						SetFont("BIOTIME3");
					
					HUDMessageUnscaled(-CIRCLE_PADDING, CIRCLE_PADDING, "a", ALIGN_CENTER, ALIGN_CENTER, HID_CIRCLE_BG, 0.0, 0.0);
					
					str numToShow = shouldFormat ? TimeString(checkNumber) : StrParam(d:checkNumber);
					
					if (!isBoss)
					{
						SetFont(GameState.State == STATE_TRADER ? "BIGFONT" : "HUGEFONT");
						HUDMessageUnscaled(-CIRCLE_PADDING, CIRCLE_PADDING, numToShow, ALIGN_CENTER, ALIGN_CENTER, HID_CIRCLE_A, 0.0, 0.0);
						
						// Bottom text (FUNNY MEME)
						// We do -1 because the last wave is a boss wave
						str botText = StrParam(d:GameState.Wave+1, s:" / ", d:MAX_WAVES-1);
						SetFont("TYPE24");
						HUDMessageUnscaled(-CIRCLE_PADDING, CIRCLE_PADDING + 40.0, botText, ALIGN_CENTER, ALIGN_CENTER, HID_CIRCLE_B, 0.0, 0.0);
					}
					else
					{
						ClearHUDMessage(HID_CIRCLE_A);
						ClearHUDMessage(HID_CIRCLE_B);
					}
				}
				else
				{
					ClearHUDMessage(HID_CIRCLE_BG);
					ClearHUDMessage(HID_CIRCLE_A);
					ClearHUDMessage(HID_CIRCLE_B);
				}
			}
			
			delay(1);
		}
	}

	// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

	// ------------------------/
	// Controls wave countdown
	// ------------------------/

	script "HUD_WaveCountdownController"
	{
		// Redraw HUD elements?
		bool dirty;
		bool wasCountdown;
		int lastTime;
		bool didBlood;
		
		ZTPrint("HUD_WaveCountdownController");
		
		while (true)
		{
			// Sync state
			if (GameState.State == STATE_COUNTDOWN && !wasCountdown)
			{
				wasCountdown = true;
				dirty = true;
				AmbientSound("wave/midwave", 127);
			}
			else if (GameState.State != STATE_COUNTDOWN && wasCountdown)
			{
				wasCountdown = false;
				dirty = true;
			}
			
			// Sync time
			if (lastTime != GameState.WaveTimer)
			{
				lastTime = GameState.WaveTimer;
				dirty = true;
			}
			
			if (dirty)
			{
				// Show something!
				if (wasCountdown)
				{
					// (fixed xPos, fixed yPos, str text, int alX, int alY, int id, fixed sTime = 3.0, fixed fTime = 0.0)
					str secText = GameState.WaveTimer == 1 ? " SECOND..." : " SECONDS...";
					str showText = StrParam(s:"WAVE ", d:GameState.Wave+1, s:" IS STARTING IN ", d:GameState.WaveTimer, s:secText);
					
					if (GameState.WaveTimer <= 3 && GameState.WaveTimer >= 1)
						LocalAmbientSound("tick/tock", 127);
					
					// Top text
					SetFont("TFNDESC");
					HUDMessageUnscaled(CenterX() * 1.0, -48.0, showText, ALIGN_CENTER, ALIGN_CENTER, HID_CDOWN_TEXT, 0.0, 0.0);
					
					// BLOOD
					if (!didBlood)
					{
						ACS_NamedExecute("HUD_WaveCountdownBlood", 0);
						didBlood = true;
					}
				}
				
				// Hide the countdown messages
				else 
				{
					didBlood = false;
					ClearHUDMessage(HID_CDOWN_TEXT);
					ClearHUDMessage(HID_CDOWN_BLOOD);
				}
				
				dirty = false;
			}
			
			Delay(1);
		}
	}

	// ------------------------/
	// Blood for countdown
	// ------------------------/

	script "HUD_WaveCountdownBlood"
	{
		for (int l=0; l<5; l++)
		{
			LocalAmbientSound("hud/bloodsplat", 127);
			SetFont(StrParam(s:"HUDBLUD", d:l+1));
			HUDMessageUnscaled(CenterX() * 1.0, -48.0, "a", ALIGN_CENTER, ALIGN_CENTER, HID_CDOWN_BLOOD, 0.0, 0.0);
			delay(10);
		}
	}
	
	//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
	
	// ------------------------/
	// Hook for persistent things
	// ------------------------/
	
	script "HUD_PersistentControl"
	{
		HUD_Init();
		
		while (true)
		{
			HUD_Draw();
			delay(1);
		}
	}
	
	// ------------------------/
	// Initialize HUD variables
	// ------------------------/
	
	private void HUD_Init()
	{
		PerkSyncTick = 999;
		
		LastWeaponIndex = -1;
		
		ResX = ResolutionX();
		ResY = ResolutionY();
		
		HUD_Generate();
	}
	
	// ------------------------/
	// Call the other draw scripts
	// ------------------------/
	
	private void HUD_Draw()
	{
		HUD_Tick();
		
		// Resolution is out of sync!
		int RX, RY;
		
		RX = ResolutionX();
		RY = ResolutionY();
		
		if (ResX != RX || ResY != RY)
		{
			ResX = RX;
			ResY = RY;
			HUD_ResChanged();
		}
		
		if (GameState.HUD.LobbyIsOpen)
			L_Draw();
			
		HUD_DrawCompass();
		
		// - - - - - - - - - - - - - - - - 
		
		// Whether or not to draw "real" HUD has changed
		if (DirtyRealState)
		{
			DirtyRealState = false;
			if (!DrawRealHUD)
			{
				HUDAmmo_Cleanup();
				HUDPerk_Cleanup();
			}
		}
		
		HUDAmmo_Draw();
		HUDPerk_Draw();
	}
	
	// ------------------------/
	// Tick event for other events
	// ------------------------/
	
	private void HUD_Tick()
	{
		int CPN = ConsolePlayerNumber();
		
		// Spectator state
		int SS = PlayerIsSpectator(CPN);
		
		// Our lobby state needs to change
		bool DirtyLobbyState = false;
		
		// Our spectator state has changed!
		if (GameState.Spectating != SS)
			DirtyLobbyState = true;
			
		// Our READY state has changed
		if (GameState.HUD.WasReady != PlayerState[CPN].Ready)
		{
			GameState.HUD.WasReady = PlayerState[CPN].Ready;
			DirtyLobbyState = true;
		}
		
		// Spectator state is dirty
		if (DirtyLobbyState)
		{
			DirtyLobbyState = false;
			GameState.Spectating = SS;
			
			bool shouldHideLobby = false;
			
			// Waiting to respawn? Don't show the lobby
			if (WaitingToRespawn())
				shouldHideLobby = true;
				
			// We're actually in the game
			if (SS == 0)
				shouldHideLobby = true;
			
			// We're IN THE GAME
			if (shouldHideLobby)
			{
				// We closed it at least once
				// This is to prevent it from re-opening when we become a spectator again
				
				if (GameState.LobbyOpen)
					GameState.HUD.ClosedLobby = true;
					
				GameState.LobbyOpen = false;
			}
				
			// We're SPECTATING
			// If we've closed our lobby menu once, never show it again
			else if (SS == 2 && !GameState.HUD.ClosedLobby)
				GameState.LobbyOpen = true;
		}
		
		bool LO = GameState.LobbyOpen;
		
		if (GameState.HUD.LobbyIsOpen != LO)
		{
			GameState.HUD.LobbyIsOpen = LO;
			
			// "Open" the menu
			if (LO)
				L_Init();
				
			// Clean it up
			else
				L_Cleanup();
		}
		
		if (GameState.HUD.LobbyIsOpen)
			L_Tick();
			
		// - - - - - - -
		
		bool HDR = HUD_DrawReal();
		
		if (DrawRealHUD != HDR)
		{
			DrawRealHUD = HDR;
			DirtyRealState = true;
		}
		
		// Tick logic for "real" HUD
		if (DrawRealHUD)
		{
			int OID = ActivatorTID();
			SetActivator( PlayerTID(CPN) );
			
			PerkSyncTick ++;
			
			// Sync indexes with player state
			// This prevents us from doing it every tick
			
			if (PerkSyncTick >= PERKSYNC_INTERVAL)
			{
				PerkSyncTick = 0;
				HUDPerk_Sync();
			}
			
			// - - - - - - -
			
			HUDAmmo_Tick();
			
			SetActivator(OID);
		}
	}
	
	// ------------------------/
	// Should we draw the HUD?
	// ------------------------/
	
	private bool HUD_DrawReal()
	{
		int CPN = ConsolePlayerNumber();
		
		return (PlayerState[CPN].Alive);
	}
	
	// ------------------------/
	// Draw compass
	// ------------------------/
	
	private void HUD_DrawCompass()
	{
		// SHOULD we draw the compass? Only if we're in the game
		bool DC = (GameState.Spectating == 0 && !GameState.HUD.TraderIsOpen);
		
		// The game has NO trader entries
		if (TraderArraySize <= 0)
			DC = false;
		
		// Mismatched state
		if (DrawCompass != DC)
		{
			DrawCompass = DC;
			
			// Stop drawing, clean it up
			if (!DC)
			{
				ClearHUDMessage(HID_COMPASS);
				ClearHUDMessage(HID_COMPASS_X);
				ClearHUDMessage(HID_COMPASS_TXT);
				ClearHUDMessage(HID_COMPASS_DIST);
			}
		}
		
		if (!DrawCompass)
			return;
		
		int TraderSpotID = TraderArray[GameState.CurrentTrader].CompassID;
		int pct;
		
		int RX = ResolutionX() * 1.0;
		
		// Get the dot product for facing the trader
		int TID = PlayerTID( ConsolePlayerNumber() );
		
		int dotX = FixedMul(RX, 0.5);
		
		// Positive means it's in FRONT of us
		// Negative means it's BEHIND us
		int FrontBack = dotFacing(TID, TraderSpotID);
		
		// Negative means it's on our RIGHT
		// Positive means it's on our LEFT
		int LeftRight = dotFacing(TID, TraderSpotID, 0.25);
		
		// It's in front of us
		if (FrontBack >= 0.0)
			pct = FixedMul(LeftRight, 0.5);
			
		// It's behind us
		else
		{
			if (LeftRight > 0.0)
				pct = 0.5;
			else
				pct = -0.5;
		}
		
		// Flip it
		pct = -pct;
		
		int cenX = CompassRect.X + 145.0;
		int comWidth = 220.0;
		int comY = CompassRect.Y + 61.0;
		
		dotX = cenX + FixedMul(comWidth, pct);
		
		HUDMessageMaterial(HUDCOMP, CompassRect, HID_COMPASS);
		
		// Draw the actual X
		SetFont("BIGFONT");
		HUDMessageUnscaled(dotX, comY, "\cEX", ALIGN_CENTER, ALIGN_CENTER, HID_COMPASS_X);
		
		// "TRADER"
		int trdY = CompassRect.Y + 17.0;
		int trdX = CompassRect.X + 17.0;
		SetFont(FNT_PLAIN);
		HUDMessageUnscaled(trdX, trdY, "TRADER", ALIGN_LEFT, ALIGN_CENTER, HID_COMPASS_TXT);
		
		trdX = CompassRect.Y + 265.0;
		
		// How far away is it?
		int myX = GetActorX(TID);
		int myY = GetActorY(TID);
		int spotX = GetActorX(TraderSpotID);
		int spotY = GetActorY(TraderSpotID);
		
		int dist = ShaveInt( FixedMul(METER_SCALE, VectorLength(spotX - myX, spotY - myY)) );
		
		SetFont(FNT_PLAIN);
		HUDMessageUnscaled(trdX, trdY, StrParam(d:dist,s:"m"), ALIGN_RIGHT, ALIGN_CENTER, HID_COMPASS_DIST);
	}
	
	// ------------------------/
	// Our resolution has changed
	// ------------------------/
	
	private void HUD_ResChanged()
	{
		HUD_Generate();
	}
	
	// ------------------------/
	// Generate HUD elements
	// ------------------------/
	
	private void HUD_Generate()
	{
		DirtyGrenade = true;
		
		// Create rectangle for compass
		int ComX = 32.0;
		int ComY = 32.0;
		
		SetRect(CompassRect, ComX, ComY, HUDCOMP.W, HUDCOMP.H);
	}
	
	//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

	// ------------------------/
	// Sync perk junk
	// ------------------------/
	
	private void HUDPerk_Sync()
	{
		int oldPerk = LastPerk;
		int oldGI = LastGrenadeIndex;
		
		LastPerk = OurPerk();
		LastGrenadeIndex = WeaponByType( PerkList[LastPerk].GrenadeClass );
		
		// Dirty the perk
		if (LastPerk != oldPerk)
			DirtyPerk = true;
		
		if (oldPerk != LastPerk || oldGI != LastGrenadeIndex)
			DirtyGrenade = true;
	}
	
	// ------------------------/
	// Check ammo
	// ------------------------/
	
	private void HUDAmmo_Tick()
	{
		int newPrim, newSec;
		
		// Check grenade count
		int gCount = GrenadesLeft();
		
		if (Ammo_LastGrenade != gCount)
		{
			Ammo_LastGrenade = gCount;
			DirtyGrenade = true;
		}
		
		// Check primary count
		newPrim = 0;
		newSec = 0;
		
		bool usesAmmo = false;
		
		// Weapon index we have
		int WI = LocalWeaponIndex();
		if (LastWeaponIndex != WI)
		{
			LastWeaponIndex = WI;
			DirtyAmmo = true;
		}
		
		if (LastWeaponIndex >= 0)
		{
			usesAmmo = true;
			newPrim = CheckInventory(WeaponList[LastWeaponIndex].PrimaryClass);
			newSec = CheckInventory(WeaponList[LastWeaponIndex].SecondaryClass);
		}
		
		// Ammo numbers mismatch
		if (Ammo_LastPrimary != newPrim || Ammo_LastSecondary != newSec)
		{
			DirtyAmmo = true;
			Ammo_LastPrimary = newPrim;
			Ammo_LastSecondary = newSec;
		}
		
		// Draw state mismatched
		if (DrawAmmoNumbers != usesAmmo)
		{
			DrawAmmoNumbers = usesAmmo;
			DirtyAmmo = true;
		}
		
		// Fire mode
		str FMI = GetFireModeImage();
		if (StrCmp(LastFMImage, FMI) != 0)
		{
			LastFMImage = FMI;
			DirtyFireMode = true;
		}
	}
	
	// ------------------------/
	// Draw ammo numbers
	// ------------------------/
	
	private void HUDAmmo_Draw()
	{
		// Dirty nade
		if (DirtyGrenade)
		{
			DirtyGrenade = false;
			SetFont(FNT_GRENADES);
			
			int nX = (ResolutionX() * 1.0) - AMMONADE_X;
			int nY = (ResolutionY() * 1.0) - AMMONADE_Y;
			HUDMessageUnscaled(nX, nY, StrParam(d:Ammo_LastGrenade), ALIGN_RIGHT, ALIGN_TOP, HID_HUD_GRENADE);
		}
		
		// Dirty ammo numbers!
		if (DirtyAmmo)
		{
			DirtyAmmo = false;
			
			if (DrawAmmoNumbers)
			{
				int aX = (ResX * 1.0) - AMMOPRIM_X;
				int aY = (ResY * 1.0) - AMMOPRIM_Y;
				
				SetFont(FNT_AMMOPRIMARY);
				HUDMessageUnscaled(aX, aY, StrParam(d:Ammo_LastPrimary), ALIGN_RIGHT, ALIGN_CENTER, HID_HUD_PRIMARY);
				
				aX = (ResX * 1.0) - AMMOSEC_X;
				aY = (ResY * 1.0) - AMMOSEC_Y;
				
				SetFont(FNT_AMMOSECONDARY);
				HUDMessageUnscaled(aX, aY, StrParam(d:Ammo_LastSecondary), ALIGN_RIGHT, ALIGN_CENTER, HID_HUD_SECONDARY);
			}
			
			else
			{
				ClearHUDMessage(HID_HUD_PRIMARY);
				ClearHUDMessage(HID_HUD_SECONDARY);
			}
		}
		
		// Fire mode
		if (DirtyFireMode)
		{
			DirtyFireMode = false;
			
			// Not in wep list
			if (LastWeaponIndex < 0)
				ClearHUDMessage(HID_HUD_FIREMODE);
			else
			{
				SetFont(LastFMImage);
				HUDMessageUnscaled((ResX*1.0) - FIREMODE_X, (ResY*1.0) - FIREMODE_Y, "a", ALIGN_CENTER, ALIGN_CENTER, HID_HUD_FIREMODE);
			}
		}
	}
	
	// ------------------------/
	// Cleanup ammo numbers
	// ------------------------/
	private void HUDAmmo_Cleanup()
	{
		ClearHUDMessage(HID_HUD_GRENADE);
		ClearHUDMessage(HID_HUD_PRIMARY);
		ClearHUDMessage(HID_HUD_SECONDARY);
		ClearHUDMessage(HID_HUD_FIREMODE);
	}
	
	//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
	
	// ------------------------/
	// Draw perk things
	// ------------------------/
	
	private void HUDPerk_Draw()
	{
		if (DirtyPerk)
		{
			DirtyPerk = false;
			
			int pX = PERKICON_X;
			int pY = (ResY*1.0) - PERKICON_Y;

			SetFont(PerkList[LastPerk].IconHUD);
			HUDMessageUnscaled(pX, pY, "a", ALIGN_CENTER, ALIGN_CENTER, HID_HUD_PERKICON);
		}
	}
	
	// ------------------------/
	// Cleanup perk things
	// ------------------------/
	
	private void HUDPerk_Cleanup()
	{
		ClearHUDMessage(HID_HUD_PERKICON);
	}
}

// ------------------------/
// FIGHT!!!
// ------------------------/

void ShowFightImage()
{
	int yPos = FixedMul(ResolutionY()*1.0, 0.6);
	SetFont("FIGHTHUD");
	HUDMessageUnscaled(CenterX() * 1.0, yPos, "a", ALIGN_CENTER, ALIGN_CENTER, HID_FIGHT, 3.0, 1.0);
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// MUSIC DISPLAY
// ------------------------/

using ZTMaterials;

void MusicDisplay_Cleanup()
{
	int cleanCount = MUSIC_SEGMENTLENGTH;
	
	// Right
	cleanCount ++;
	
	// Left
	cleanCount ++;
	
	for (int i=0; i<cleanCount; i++)
		ClearHUDMessage(HID_MUSIC_BG + i);
		
	// Text
	ClearHUDMessage(HID_MUSIC_SONG);
	ClearHUDMessage(HID_MUSIC_ARTIST);
}

void ShowTrackInfo(str tName, str tArtist)
{
	MusicDisplay_Cleanup();
	ACS_NamedTerminate("MusicDisplay", 0);
	
	GameState.HUD.CurrentSong = tName;
	GameState.HUD.CurrentArtist = tArtist;
	
	ACS_NamedExecuteAlways("MusicDisplay", 0);
}

// ------------------------/
// ACTUAL HANDLER FOR MUSIC DISPLAY
// ------------------------/

script "MusicDisplay"
{
	int HID;
	int FadeCount;					// How many times we've faded
	
	int MusX = MUSIC_STARTX;
	int MusY = MUSIC_STARTY;
	
	// Initial pulses
	while (FadeCount < MUSIC_FADEGOAL)
	{
		SetFont("KFMUFADE");
		HUDMessageUnscaled(MusX, MusY, "a", ALIGN_LEFT, ALIGN_TOP, HID_MUSIC_BG, MUSIC_SHOWTIME, MUSIC_FADETIME, HUDMSG_FADEINOUT);
		
		int waitTime = MUSIC_SHOWTIME + MUSIC_FADETIME + MUSIC_FADETIME;
		
		delay(ShaveInt(FixedMul(35.0, waitTime)));
		FadeCount ++;
	}
	
	// Now is a great time to show our track info
	int songX = MusX + KFMULEFT.W + MUSIC_SONGX;
	int songY = MusY + MUSIC_SONGY;
	int artX = MusX + KFMULEFT.W + MUSIC_ARTISTX;
	int artY = MusY + MUSIC_ARTISTY;
	
	SetFont(FNT_MUSICSONG);
	HUDMessageUnscaled(songX, songY, GameState.HUD.CurrentSong, ALIGN_LEFT, ALIGN_CENTER, HID_MUSIC_SONG, MUSIC_INFO_HOLD, MUSIC_INFO_FADEIN, HUDMSG_FADEINOUT, MUSIC_INFO_FADEOUT);
	
	SetFont(FNT_MUSICARTIST);
	HUDMessageUnscaled(artX, artY, GameState.HUD.CurrentArtist, ALIGN_LEFT, ALIGN_CENTER, HID_MUSIC_ARTIST, MUSIC_INFO_HOLD, MUSIC_INFO_FADEIN, HUDMSG_FADEINOUT, MUSIC_INFO_FADEOUT);
	
	// Now let's scale it up
	HID = HID_MUSIC_BG;
	
	int SegmentCount;				// Segments we've created so far
	int x;
	
	int plainTID = HID;
	
	while (SegmentCount < MUSIC_SEGMENTLENGTH)
	{
		// Last message in the loop?
		bool isLast = (SegmentCount + MUSIC_SEGMENTINCREASE >= MUSIC_SEGMENTLENGTH);
		int sTime = isLast ? 3.0 : 0.0;
		int fTime = isLast ? 1.0 : 0.0;
		
		HID = plainTID;
		x = MusX;
		
		SetFont("KFMULEFT");
		HUDMessageUnscaled(x, MusY, "a", ALIGN_LEFT, ALIGN_TOP, HID, sTime, fTime);
		HID ++;
		x += KFMULEFT.W;
		
		// Loop through segments
		for (int i=0; i<SegmentCount; i++)
		{
			SetFont("KFMUCENT");
			HUDMessageUnscaled(x, MusY, "a", ALIGN_LEFT, ALIGN_TOP, HID, sTime, fTime);
			HID ++;
			x += KFMUCENT.W;
		}
		
		// Right
		SetFont("KFMURIGH");
		HUDMessageUnscaled(x, MusY, "a", ALIGN_LEFT, ALIGN_TOP, HID, sTime, fTime);
		HID ++;
		
		delay(MUSIC_SEGMENTTICS);
		
		// Extra fast
		SegmentCount += MUSIC_SEGMENTINCREASE;
	}
}