// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - UI
// Contains UI functions
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#define AV_CENTER 			0.4
#define AV_CENTER_Y 		0.0
#define AV_LEFT 			0.1
#define AV_RIGHT 			0.2

#define X_CENTER			1.5
#define Y_CENTER			0.5

// Accessor for a "picture" - Has dimensions and a texture
struct Picture
{
	int W, H;
	str Tex;
};

void CopyPicture(struct Picture& PicSrc, struct Picture& PicDest)
{
	PicDest.W = PicSrc.W;
	PicDest.H = PicSrc.H;
	PicDest.Tex = PicSrc.Tex;
}

// Bounding box
struct Rect
{
	int X, Y;
	int W, H;
};

// Set a rectangle's dimensions
void SetRect(struct Rect& Box, int X, int Y, int W, int H)
{
	Box.X = X;
	Box.Y = Y;
	Box.W = W;
	Box.H = H;
}

// Alignment for text and such
enum Alignment
{
	ALIGN_LEFT = 0,
	ALIGN_TOP = 0,
	ALIGN_CENTER = 1,
	ALIGN_RIGHT = 2,
	ALIGN_BOTTOM = 2,
};

// Position + Alignment = Final Pos
int AliX(int pos, int al) { return pos + ResolveAlignment(al, false); }
int AliY(int pos, int al) { return pos + ResolveAlignment(al, true); }

// Get the number to ADD based on the alignment value
int ResolveAlignment(int num, bool vertical = false)
{
	switch (num)
	{
		// Left / Top
		case ALIGN_LEFT:
			return AV_LEFT;
		break;
		
		// Center
		case ALIGN_CENTER:
			return vertical ? AV_CENTER_Y : AV_CENTER;
		break;
		
		// Right / Bottom
		case ALIGN_RIGHT:
			return AV_RIGHT;
		break;
	}
	
	return 0.0;
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Half-res of screen
int CenterX()
{
	return ResolutionX() / 2;
}

// Half-res of screen
int CenterY()
{
	return ResolutionY() / 2;
}

// Obtain X resolution
int ResolutionX()
{
	if (IsServer())
		return PlayerState[ PlayerNumber() ].ResX;
		
	// Use the cached value
	return GameState.LastResX;
}

// Obtain Y resolution
int ResolutionY()
{
	if (IsServer())
		return PlayerState[ PlayerNumber() ].ResY;
		
	// Use the cached value
	return GameState.LastResY;
}

// ------------------------/
// Begin syncing resolution-related things
// ------------------------/

void StartResolutionHook()
{
	// Resolution cannot be grabbed on dedicated servers
	if (IsServer())
		return;
		
	// Begin our resolution handler script
	ACS_NamedExecute("ResolutionHandler",0,0,0,0);
}

// ------------------------/
// ResolutionHandler
// Sends our res to the server if necessary
// ------------------------/

script "ResolutionHandler" CLIENTSIDE
{
	while (true)
	{
		int ResX = GetScreenWidth();
		int ResY = GetScreenHeight();
		
		// They're different! Let the server know
		if (ResX != GameState.LastResX || ResY != GameState.LastResY)
		{
			// Update them locally
			GameState.LastResX = ResX;
			GameState.LastResY = ResY;
			
			// RELIABLY send them to the server
			ACS_NamedExecute("ResolutionSender",0,ResX,ResY,0);
		}
		
		delay(RES_CHECK_INTERVAL);
	}
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Clears a HUD message
// ------------------------/

void ClearHUDMessage(int id)
{
	HudMessage(s:""; HUDMSG_PLAIN, id, CR_UNTRANSLATED, 0.0, 0.0, 0.2);
}

// ------------------------/
// Clears a RANGE of HUD messages
// ------------------------/

void ClearHUDMessageRange(int id, int count, bool backwards = true)
{
	int counter = id;
	
	for (int c=0; c<count; c++)
	{
		ClearHUDMessage(counter);
		counter += backwards ? -1 : 1;
	}
}

// ------------------------/
// HUDMessageUnscaled
// Shows an UNSCALED message on the screen
//
// xPos, yPos: 			X and Y coordinates (as fixed)
// text: 				The text to show
// alX, alY:			X and Y alignment values
// id:					ID to use for it
// sTime, fTime:		Show / Fade times
// ------------------------/

void HUDMessageUnscaled(fixed xPos, fixed yPos, str text, int alX, int alY, int id, fixed sTime = 0.0, fixed fTime = 0.0, int messType = -1, fixed fOutTime = -1.0)
{
	int resX, resY, finalX, finalY, type, alignmentX, alignmentY, foTi;
	
	// Resolve alignment based on our enum
	alignmentX = ResolveAlignment(alX, false);
	alignmentY = ResolveAlignment(alY, true);
	
	// Convert to floats
	xPos = ToFloat(xPos);
	yPos = ToFloat(yPos);
	
	// Decide our screen resolution
	resX = ResolutionX() * 1.0;
	resY = ResolutionY() * 1.0;
	
	// Wrap them around if they're negative
	xPos = (xPos < 0.0) ? resX + xPos : xPos;
	yPos = (yPos < 0.0) ? resY + yPos : yPos;

	// If showTime is 0.0, it should fade out
	type = (sTime > 0.0) ? (HUDMSG_PLAIN | HUDMSG_FADEOUT) : HUDMSG_PLAIN;
	
	// Type is ACTUALLY DEFINED
	if (messType >= 0)
		type = messType;
		
	foTi = fTime;
	if (fOutTime >= 0.0)
		foTi = fOutTime;

	SetHUDSize(ShaveInt(resX), ShaveInt(resY), false);
	HudMessage(s:text; type, id, CR_UNTRANSLATED, xPos + alignmentX, yPos + alignmentY, sTime, fTime, foTi);
	SetHUDSize(0, 0, false);
}

// ------------------------/
// Draws an image using a PICTURE and a RECT
// ------------------------/

void HUDMessageMaterial(struct Picture& Pic, struct Rect& Rectangle, int ID, int aX = ALIGN_LEFT, int aY = ALIGN_TOP, str forcePic = "")
{
	SetFont((forcePic == "") ? Pic.Tex : forcePic);
	HUDMessageUnscaled(Rectangle.X, Rectangle.Y, "a", aX, aY, ID);
}

// ------------------------/
// Draws an image using a PICTURE, no RECT
// ------------------------/

void HUDMessageMaterialOnly(struct Picture& Pic, int X, int Y, int ID, int aX = ALIGN_LEFT, int aY = ALIGN_TOP, str forcePic = "")
{
	SetFont((forcePic == "") ? Pic.Tex : forcePic);
	HUDMessageUnscaled(X, Y, "a", aX, aY, ID);
}

// ------------------------/
// Draw a SEGMENTED HUD message
// This attemps to vertically fill a rectangle with images
//
// TOP - Top image to use as a Picture
// MID - Middle image to use as a Picture
// BOTTOM - Bottom image to use as a Picture
// 
// Returns a brand new ID after having drawn everything
// ------------------------/

int HUDMessageRectangle(int X, int Y, int H, int ID, struct Picture& Top, struct Picture& Mid, struct Picture& Bottom, bool clear = false)
{
	int curY;
	int curID = ID;
	
	// Begin with middle pictures first
	curY = Y+Top.H;
	while (curY < (Y+H) - Bottom.H)
	{
		// Null pic
		if (clear)
			ClearHUDMessage(curID);
		else
			HUDMessageMaterialOnly(Mid, X, curY, curID);
				
		curY += Mid.H;
		curID --;
	}
	
	// Top pic!
	if (clear)
		ClearHUDMessage(curID);
	else
		HUDMessageMaterialOnly(Top, X, Y, curID);
	curID --;
	
	// Bottom pic!
	if (clear)
		ClearHUDMessage(curID);
	else
		HUDMessageMaterialOnly(Bottom, X, (Y+H)-Bottom.H, curID);
	curID --;
	
	return curID;
}

// ------------------------/
// Draw a SEGMENTED HUD message (but horizontally!)
// This attemps to HORIZONTALLY fill a rectangle with images
//
// TOP - Top image to use as a Picture
// MID - Middle image to use as a Picture
// BOTTOM - Bottom image to use as a Picture
// 
// Returns a brand new ID after having drawn everything
// ------------------------/

int HUDMessageRectangleHor(int X, int Y, int W, int ID, struct Picture& Top, struct Picture& Mid, struct Picture& Bottom, bool clear = false)
{
	int curX;
	int curID = ID;
	
	// Begin with middle pictures first
	curX = X+Top.W;
	while (curX < (X+W) - Bottom.W)
	{
		// Null pic
		if (clear)
			ClearHUDMessage(curID);
		else
			HUDMessageMaterialOnly(Mid, curX, Y, curID);
				
		curX += Mid.W;
		curID --;
	}
	
	// Top pic!
	if (clear)
		ClearHUDMessage(curID);
	else
		HUDMessageMaterialOnly(Top, X, Y, curID);
	curID --;
	
	// Bottom pic!
	if (clear)
		ClearHUDMessage(curID);
	else
		HUDMessageMaterialOnly(Bottom, (X+W)-Bottom.W, Y, curID);
	curID --;
	
	return curID;
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Get map title
// ------------------------/

str GetMapTitle()
{
	if (StrLen(GameState.MapTitle) > 0)
		return GameState.MapTitle;
	
	// Fallback to map name
	return StrParam(n:PRINTNAME_LEVELNAME);
}

// ------------------------/
// Get map author
// ------------------------/

str GetMapAuthor()
{
	if (StrLen(GameState.MapAuthor) > 0)
		return GameState.MapAuthor;
		
	return "Unknown";
}

// ------------------------/
// Get map image
// ------------------------/

str GetMapPicture()
{
	if (StrLen(GameState.MapPicture) > 0)
		return GameState.MapPicture;
		
	return "MPICNONE";
}

// ------------------------/
// Get map description
// ------------------------/

str GetMapDescription()
{
	if (StrLen(GameState.MapDescription) > 0)
		return GameState.MapDescription;
		
	return "No description has been provided.";
}

// ------------------------/
// Set info about the map!
// ------------------------/

void SetMapInformation(str mapName, str mapAuthor, str mapPic = "MPICNONE", str mapDescription = "No description has been provided.")
{
	GameState.MapTitle = mapName;
	GameState.MapAuthor = mapAuthor;
	GameState.MapPicture = mapPic;
	GameState.MapDescription = mapDescription;
}