// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - ZOMBIES
// Contains core functionality for all zombies
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// Initialize the zombie, set core parameters
void InitializeZombie()
{
	// Already initialize
	if (CheckInventory("ObjectReady"))
		return;
		
	GiveInventory("IsZT2Monster", 1);
	GiveInventory("ObjectReady", 1);
	
	// Find our index in the table
	int ZI = ZombieByType( GetActorClass(0) );
	int FinalIndex = (ZI < 0) ? 0 : ZI + ZLIST_BASE;
	
	// Our final index, cache it as an inventory item
	// Adding ZLIST_BASE to it lets us still check for 0 of the item
	// (Distinguishes an actor that has 0 of the item from a zombie at index 0)
	
	InvEnsure("ZombieIndex", FinalIndex);
	
	HatePlayer();
}

script "InitializeZombie" (void) { InitializeZombie(); }

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Get a number of all LIVING zombies on the map
// ------------------------/

int AliveZombies()
{
	int count = 0;
	
	for (int i=0; i<ZombieListSize; i++)
		count += ThingCountName(ZombieList[i].Class, 0);
		
	return count;
}

// ------------------------/
// All zombies are dead!
// Killed >= Goal
// ------------------------/

bool AllZombiesKilled()
{
	// Boss?
	if (IsBossWave())
		return false;
		
	return (GameState.ZombiesKilled >= GameState.ZombieGoal);
}

// ------------------------/
// Number of zombies that are "left"
// ------------------------/

int RemainingZombies()
{
	return GameState.ZombieGoal - GameState.ZombiesKilled;
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// Cleanup zombies that are stuck
// (Janitorial work for Spawner Director)
// ------------------------/

void CleanupLostZombies()
{
	if (GameState.State != STATE_WAVE)
		return;
		
	if (AllZombiesKilled())
		CompleteWave();
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// ------------------------/
// A zombie was killed!
// ------------------------/

script "ZombieKilled" (void) { ZombieKilled(); }
void ZombieKilled()
{
	// Already killed, don't worry about it
	if (CheckInventory("Deceased"))
		return;
		
	// Give us a unique ID
	int OID = ActivatorTID();
	int UID = UniqueTID();
	
	Thing_ChangeTID(0, UID);
		
	// How much money should we give?
	int ZI = GetZombieIndex();
	int moneyAmt = ZombieList[ZI].Bounty[BOUNTY_MAIN];
	int xpAmt = ZombieList[ZI].XP[BOUNTY_MAIN];
	
	// Decapitated, gross
	if (CheckInventory("Decapitated"))
	{
		moneyAmt += ZombieList[ZI].Bounty[BOUNTY_HEADSHOT];
		xpAmt += ZombieList[ZI].XP[BOUNTY_HEADSHOT];
	}
		
	// Set to player
	SetActivatorToTarget(0);

	// Player?
	if (ClassifyActor(0) & ACTOR_PLAYER)
	{
		GiveInventory("Money", moneyAmt);
		GiveXP(xpAmt);
	}
		
	// Set back to zombie
	SetActivator(UID);
	Thing_ChangeTID(0, OID);
	
	GiveInventory("Deceased", 1);

	SetActivator(-1);

	// Don't rely on AliveZombies quite yet! We need to play our death anim
	GameState.ZombiesKilled ++;
	
	// Send the new number to all players, don't calculate it
	UpdateZombiesLeft();
}

// ------------------------/
// Hate a player automatically!
// TODO: THIS CAN PROBABLY BE OPTIMIZED
// ------------------------/

void HatePlayer()
{
	int storedPlayerTID;
	int lastDistance = 99999;
	
	for (int l=0; l<MAXPLAYERS; l++)
	{
		if (!PlayerInGame(l))
			continue;
			
		int TID = BASE_PLAYER_TID + l;
			
		int LD = lazyDistance(0, TID);
		if (LD < lastDistance)
		{
			storedPlayerTID = TID;
			lastDistance = LD;
		}
	}
	
	if (storedPlayerTID <= 0)
		return;
		
	Thing_Hate(0, storedPlayerTID);
}

// ------------------------/
// Kill all zombies!
// ------------------------/

void KillZombies()
{
	for (int i=0; i<ZombieListSize; i++)
		Thing_Destroy(ZombieList[i].TID, false, 0);
}

// ------------------------/
// The boss died
// (Boss as activator)
// ------------------------/

script "BossKilled"
{
	int x = GetActorX(0);
	int y = GetActorY(0);
	int z = GetActorZ(0);
	int ang = GetActorAngle(0);
	
	InvulAllPlayers();
	
	int cam = SpawnCinematicCameraForced(x, y, z, ang, ActivatorTID());
	if (cam)
		ACS_NamedExecute("BossCinematic", 0, cam, 0);
}